#include <stdio.h>
#include <stdlib.h>


#include "ReadWriteFeat.h"

/* SwapInt: swap byte order of int(32) data value *p */
static void SwapInt(int *p)
{
	char temp,*q;
	
	q = (char*) p;
	temp = *q; *q = *(q+3); *(q+3) = temp;
	temp = *(q+1); *(q+1) = *(q+2); *(q+2) = temp;
}

/* SwapShort: swap byte order of short data value *p */
static void SwapShort(short *p)
{
	char temp,*q;
	
	q = (char*) p;
	temp = *q; *q = *(q+1); *(q+1) = temp;
}

/* Read file using reverse byte order (return 1 if 
   all bytes are correctly read) */
static int ReadReverse(void *a, int size, FILE *fp)
{
	int i, n;
	char c, *p;
	
	n = 0;
	p = a+size-1;
	for (i = 0; i < size; i++, p--) {
		n += fread(&c, 1, 1, fp);
		*p = c;
	}
	
	if (n == size)
		return 1;
	else 
		return 0;
}

static int WriteReverse(void *a, int size, FILE *fp) 
{
	int i, n;
	char *p;

	n = 0;
	p = a+size-1;
	for (i = 0; i < size; i++, p--) {
		n += fwrite(p, 1, 1, fp);
	}

	if (n == size)
		return 1;
	else 
		return 0;
}

static void LifterGslVec(gsl_vector *vec, int ceplifter)
{
    int d;
    float scalefac;
    int numceps = vec->size;

    /* Note: Nothing is done with C0 */
    for (d = 1; d < numceps; d++) {
		scalefac = (1 + ceplifter*sin(M_PI*d/ceplifter)/2.0);
		gsl_vector_set(vec, d, gsl_vector_get(vec, d)*scalefac);
    }
}

static void UnLifterGslVec(gsl_vector *vec, int ceplifter)
{
    int d;
    float scalefac;
    int numceps = vec->size;

    /* Note: Nothing is done with C0 */
    for (d = 1; d < numceps; d++) {
		scalefac = (1 + ceplifter*sin(M_PI*d/ceplifter)/2.0);
		gsl_vector_set(vec, d, gsl_vector_get(vec, d)/scalefac);
    }
}

void DoUnLiftering(gsl_matrix *data, int ceplifter)
{
    int t;
    int frames = data->size1;

   	gsl_vector_view vv;
	gsl_vector *yt;

    for (t = 0; t < frames; t++) {
   		vv = gsl_matrix_row(data, t);
		yt = &vv.vector;        
    
        UnLifterGslVec(yt, ceplifter);
    }
    
}

void DoLiftering(gsl_matrix *data, int ceplifter)
{
    int t;
    int frames = data->size1;

   	gsl_vector_view vv;
	gsl_vector *yt;

    for (t = 0; t < frames; t++) {
   		vv = gsl_matrix_row(data, t);
		yt = &vv.vector;        
    
        LifterGslVec(yt, ceplifter);
    }
    
}

/* 
	Note: revByteOrder must be set (contrary to LoadFeat, 
	where it is found).
*/
int WriteFeat(const char *fn, Header hdr, int revByteOrder, float **data)
{
	FILE *fp;
	int i, j, n;


	fp = fopen(fn, "wb");
	if (fp == NULL) {
		return 1;
	}

	if (revByteOrder) {
		SwapInt(&hdr.nSamples);
		SwapInt(&hdr.sampPeriod);
		SwapShort(&hdr.sampSize);
		SwapShort(&hdr.parmKind);
	}

	if (fwrite(&hdr.nSamples, 4, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.sampPeriod, 4, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.sampSize, 2, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.parmKind, 2, 1, fp) != 1) {
		return 2;
	}

	if (revByteOrder) {
		/* swap byte order back so that values can be used */
		SwapInt(&hdr.nSamples);
		SwapInt(&hdr.sampPeriod);
		SwapShort(&hdr.sampSize);
		SwapShort(&hdr.parmKind);
	}

	for (i = 0; i < hdr.nSamples; i++) {
		for (j = 0; j < hdr.sampSize/sizeof(float); j++) {
			if (revByteOrder)
				n = WriteReverse(&data[i][j], 4, fp);
			else
				n = fwrite(&data[i][j], 4, 1, fp);

			if (n != 1) {
				/* error writing */
				return 3;
			}
		}
	}
	
	fclose(fp);
	
	return 0;
}

int WriteCepstrumGsl(const char *fn, Header hdr, int revByteOrder, gsl_matrix *feat)
{
    FILE *fp;
	int i, j, n;
    float temp;

    if (hdr.parmKind != MFCC_0) {
        printf("Error: parmKind must be MFCC_0!\n");
        return 4;
    }

    /* check that there is consistency between feature dimension and sampSize */
    if (hdr.sampSize/sizeof(float) != feat->size2) {
        printf("Feature dimension error: sampSize = %d floats while dim = %d.\n", (int)(hdr.sampSize/sizeof(float)), (int)feat->size2);
        return 5;
    }

	fp = fopen(fn, "wb");
	if (fp == NULL) {
		return 1;
	}

	if (revByteOrder) {
		SwapInt(&hdr.nSamples);
		SwapInt(&hdr.sampPeriod);
		SwapShort(&hdr.sampSize);
		SwapShort(&hdr.parmKind);
	}

	if (fwrite(&hdr.nSamples, 4, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.sampPeriod, 4, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.sampSize, 2, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.parmKind, 2, 1, fp) != 1) {
		return 2;
	}

	if (revByteOrder) {
		/* swap byte order back so that values can be used */
		SwapInt(&hdr.nSamples);
		SwapInt(&hdr.sampPeriod);
		SwapShort(&hdr.sampSize);
		SwapShort(&hdr.parmKind);
	}

	for (i = 0; i < feat->size1; i++) {
        // start with element 1 and write C0 last 
		for (j = 1; j < feat->size2; j++) {
            temp = (float)gsl_matrix_get(feat, i, j);
			if (revByteOrder)
				n = WriteReverse(&temp, 4, fp);
			else
				n = fwrite(&temp, 4, 1, fp);

			if (n != 1) {
				/* error writing */
				return 3;
			}
		}
        temp = (float)gsl_matrix_get(feat, i, 0);
        if (revByteOrder)
			n = WriteReverse(&temp, 4, fp);
		else
			n = fwrite(&temp, 4, 1, fp);

		if (n != 1) {
			/* error writing */
			return 3;
        }
	}
	
	fclose(fp);
	
	return 0;

}

/*
    Same as WriteFeat but with a gsl_matrix as input.
*/
int WriteFeatGsl(const char *fn, Header hdr, int revByteOrder, gsl_matrix *feat)
{
    FILE *fp;
	int i, j, n;
    float temp;

    /* check that there is consistency between feature dimension and sampSize */
    if (hdr.sampSize/sizeof(float) != feat->size2) {
        printf("Feature dimension error: sampSize = %d floats while dim = %d.\n", (int)(hdr.sampSize/sizeof(float)), (int)feat->size2);
        return 5;
    }

	fp = fopen(fn, "wb");
	if (fp == NULL) {
		return 1;
	}

	if (revByteOrder) {
		SwapInt(&hdr.nSamples);
		SwapInt(&hdr.sampPeriod);
		SwapShort(&hdr.sampSize);
		SwapShort(&hdr.parmKind);
	}

	if (fwrite(&hdr.nSamples, 4, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.sampPeriod, 4, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.sampSize, 2, 1, fp) != 1) {
		return 2;
	}
	if (fwrite(&hdr.parmKind, 2, 1, fp) != 1) {
		return 2;
	}

	if (revByteOrder) {
		/* swap byte order back so that values can be used */
		SwapInt(&hdr.nSamples);
		SwapInt(&hdr.sampPeriod);
		SwapShort(&hdr.sampSize);
		SwapShort(&hdr.parmKind);
	}

	for (i = 0; i < feat->size1; i++) {
        // start with element 1 and write C0 last 
		for (j = 0; j < feat->size2; j++) {
            temp = (float)gsl_matrix_get(feat, i, j);
			if (revByteOrder)
				n = WriteReverse(&temp, 4, fp);
			else
				n = fwrite(&temp, 4, 1, fp);

			if (n != 1) {
				/* error writing */
				return 3;
			}
		}
	}
	
	fclose(fp);
	
	return 0;

}

int readHeader(const char *fn, Header *hdr, int *revByteOrder)
{
    FILE *fp;
 	int nSamples;
	int sampPeriod;
	short sampSize;
	short parmKind;

	*revByteOrder = 0;

	nSamples = 0;
	sampPeriod = 0;
	sampSize = 0;
	parmKind = 0;

	fp = fopen(fn, "rb");
	if (fp == NULL) {
		return 1;
	}
	/* number of samples */
	if (fread(&nSamples, 4, 1, fp) != 1) {
		return 2;
	}
	/* sample period */
	if (fread(&sampPeriod, 4, 1, fp) != 1) {
		return 2;
	}
	/* sample size, in bytes */
	if (fread(&sampSize, 2, 1, fp) != 1) {
		return 2;
	}
	/* code indicating sample kind */
	if (fread(&parmKind, 2, 1, fp) != 1) { 
		return 2;
	}

	/* are these values valid? if not check byte order */
	if (sampSize <= 0 || sampSize > 5000 || nSamples <= 0 ||
	    sampPeriod <= 0 || sampPeriod > 1000000) {
		/* try to reverse byte order */
		SwapInt(&nSamples);
		SwapInt(&sampPeriod);
		SwapShort(&sampSize);
		SwapShort(&parmKind);
		/* still a problem? in that case we quit, otherwise we
		   set revByteOrder to 1 */
		if (sampSize <= 0 || sampSize > 5000 || nSamples <= 0 ||
			sampPeriod <= 0 || sampPeriod > 1000000) 
			return 3;
		else
			*revByteOrder = 1;
	}   

	/* set the header information (if hdr != NULL) */
	if (hdr != NULL) {
		hdr->nSamples = nSamples;
		hdr->sampPeriod = sampPeriod;
		hdr->sampSize = sampSize;
		hdr->parmKind = parmKind;
	}

    fclose(fp);

    return 0;
}

/* 
	If the input file is not read successfully on the first attempt, 
	LoadFeat tries to reverse the byteorder. If this works, 
	revByteOrder is set to 1. (By default it is set to 0.)
*/
float **LoadFeat(const char *fn, int vecsToRead, Header *hdr, int *revByteOrder)
{
	FILE *fp;
	int i, j, n;
	float **featVecs;
	float temp;
	int error = 0;

    if (readHeader(fn, hdr, revByteOrder)) {
        return NULL;
    }
    
   	fp = fopen(fn, "rb");
	if (fp == NULL) {
		return NULL;
	}

#ifdef PRINTINFO
	printf("nSamples   = %d\n", hdr->nSamples);
	printf("sampPeriod = %d\n", hdr->sampPeriod);
	printf("sampSize   = %d\n", hdr->sampSize);
	printf("parmKind   = %d\n", hdr->parmKind);
#endif
	
	/* if vecsToRead is -1, read all samples */
	if (vecsToRead == -1) {
		vecsToRead = hdr->nSamples;
	}
	
	featVecs = (float **)malloc(vecsToRead*sizeof(float *));
	
    if(fseek(fp, 12L, SEEK_SET))
        return NULL;
    
	for (i = 0; i < vecsToRead; i++) {
		featVecs[i] = (float *)malloc(hdr->sampSize);
		for (j = 0; j < hdr->sampSize/sizeof(float); j++) {
			if (*revByteOrder) 
				n = ReadReverse(&temp, 4, fp);
			else
				n = fread(&temp, 4, 1, fp);

			if (n != 1) {
				/* error reading */
				error = i+1; 
				break;
			}
	    
			featVecs[i][j] = temp;
		}
		if (error) 
			break;
	}
	
	fclose(fp);

	if (error) {
		for (i = 0; i < error; i++) {
			free(featVecs[i]);
		}
		free(featVecs);
		return NULL;
	}

	return featVecs;
}

/*
    Same as LoadFeat but with a gsl_matrix as result.
*/
gsl_matrix *LoadFeatGsl(const char *fn, int vecsToRead, Header *hdr, int *revByteOrder)
{
	FILE *fp;
	int i, j, n;
	gsl_matrix *featVecs;
	float temp;
	int error = 0;
    int dim;
    int numel;

    if (readHeader(fn, hdr, revByteOrder)) {
        return NULL;
    }
    
   	fp = fopen(fn, "rb");
	if (fp == NULL) {
		return NULL;
	}
	
	/* if vecsToRead is -1, read all samples */
	if (vecsToRead == -1) {
		vecsToRead = hdr->nSamples;
	}
	
    numel = hdr->sampSize/sizeof(float);
    dim = numel;
    
	featVecs = gsl_matrix_calloc(vecsToRead, dim);
	
    if(fseek(fp, 12L, SEEK_SET))
        return NULL;
    
	for (i = 0; i < vecsToRead; i++) {
		for (j = 0; j < numel; j++) {
			if (*revByteOrder) 
				n = ReadReverse(&temp, 4, fp);
			else
				n = fread(&temp, 4, 1, fp);

			if (n != 1) {
				// error reading 
				error = i+1; 
				break;
			}
            
            gsl_matrix_set(featVecs, i, j, (double)temp);
        }
		if (error) 
			break;
	}
	
	fclose(fp);

	if (error) {
		gsl_matrix_free(featVecs);
		return NULL;
	}

	return featVecs;
}

/*
    This function reads feature file, checks that parmKind is either MFCC_0, or MFCC_E_0.
    If the type is MFCC_E_0, the energy is ignored. C0 is put first in each vector.
*/
gsl_matrix *LoadCepstrumGsl(const char *fn, int vecsToRead, Header *hdr, int *revByteOrder)
{
	FILE *fp;
	int i, j, n;
	gsl_matrix *featVecs;
	float temp;
	int error = 0;
    int dim;
    int numel;

    if (readHeader(fn, hdr, revByteOrder)) {
        return NULL;
    }
    
    /*** 
        Check that parmKind is correct. We want either MFCC_0 (8198: MFCC=6, _0=020000)
        or MFCC_E_0 (8262: MFCC=6, _E_0 = 020100).
    ***/
    
    if (hdr->parmKind != MFCC_0 && hdr->parmKind != MFCC_E_0) {
        printf("Error: parmKind must be either MFCC_0 or MFCC_E_0! parmKind = %d\n", hdr->parmKind);
        return NULL;
    }

   	fp = fopen(fn, "rb");
	if (fp == NULL) {
		return NULL;
	}
	
	/* if vecsToRead is -1, read all samples */
	if (vecsToRead == -1) {
		vecsToRead = hdr->nSamples;
	}
	
    numel = hdr->sampSize/sizeof(float);
    if (hdr->parmKind == MFCC_0)
        dim = numel;
    else
        dim = numel-1; // we do not want to allocate space for energy
    
	featVecs = gsl_matrix_calloc(vecsToRead, dim);
	
    if(fseek(fp, 12L, SEEK_SET))
        return NULL;
    
	for (i = 0; i < vecsToRead; i++) {
		for (j = 0; j < numel; j++) {
			if (*revByteOrder) 
				n = ReadReverse(&temp, 4, fp);
			else
				n = fread(&temp, 4, 1, fp);

			if (n != 1) {
				// error reading 
				error = i+1; 
				break;
			}
            
            if (hdr->parmKind == MFCC_E_0) {
                // note that nothing is done for j == numel-1, since this is energy
                if (j < numel-2) {
                    gsl_matrix_set(featVecs, i, j+1, (double)temp);
                } else if (j == numel-2) {
                    gsl_matrix_set(featVecs, i, 0, (double)temp); // this is C0
                }
            } else {
                if (j < numel-1) {
                    gsl_matrix_set(featVecs, i, j+1, (double)temp);
                } else {
                    gsl_matrix_set(featVecs, i, 0, (double)temp); // this is C0
                }
            }
        }
		if (error) 
			break;
	}
	
	fclose(fp);

	if (error) {
		gsl_matrix_free(featVecs);
		return NULL;
	}

	return featVecs;
}

int PrintCepstrum(const char *fn, int vecsToPrint, int ceplifter)
{
    gsl_matrix *data;
    Header hdr;
    int rbo;
    int n, d;
    
    data = LoadCepstrumGsl(fn, vecsToPrint, &hdr, &rbo);

    if (data == NULL) {
        printf("Error reading features from file %s.\n", fn);
        return 1;
    }
    
    if (ceplifter)
        DoUnLiftering(data, ceplifter);
    
    for (n = 0; n < data->size1; n++) {
        for (d = 0; d < data->size2; d++) {
            printf("%.3f", gsl_matrix_get(data, n, d));
            if (d < data->size2-1)
                printf(" ");
        }
        printf("\n");
    }
    
    gsl_matrix_free(data);
    
    return 0;
}
