#ifndef _READWRITEFEAT_H
#define _READWRITEFEAT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <gsl/gsl_matrix.h>
#include <gsl/gsl_vector.h>

#define MFCC_0   8198
#define MFCC_E_0 8262

typedef struct {
	int nSamples;
	int sampPeriod;
	short sampSize;
	short parmKind;
} Header;

int readHeader(const char *fn, Header *hdr, int *revByteOrder);

/* 
	Note that for LoadFeat revByteOrder is set to 0 or 1 depending on 
	whether the byte order had to be reversed to be able to read the 
	feature file.
*/
float **LoadFeat(const char *fn, int vecsToRead, Header *hdr, int *revByteOrder);

/* Note that byte order must be set when calling WriteFeat. */
int WriteFeat(const char *fn, Header hdr, int revByteOrder, float **data);

/*
    Same as LoadFeat but with a gsl_matrix as result.
*/
gsl_matrix *LoadFeatGsl(const char *fn, int vecsToRead, Header *hdr, int *revByteOrder);

/*
    Same as WriteFeat but with a gsl_matrix as input.
*/
int WriteFeatGsl(const char *fn, Header hdr, int revByteOrder, gsl_matrix *feat);

/*
    This function reads feature file, checks that parmKind is either MFCC_0, or MFCC_E_0.
    If the type is MFCC_E_0, the energy is ignored. C0 is put first in each vector.
*/
gsl_matrix *LoadCepstrumGsl(const char *fn, int vecsToRead, Header *hdr, int *revByteOrder);

int WriteCepstrumGsl(const char *fn, Header hdr, int revByteOrder, gsl_matrix *feat);

void DoUnLiftering(gsl_matrix *data, int ceplifter);
void DoLiftering(gsl_matrix *data, int ceplifter);
int PrintCepstrum(const char *fn, int vecsToPrint, int ceplifter);

#ifdef __cplusplus
}
#endif

#endif // _READWRITEFEAT_H
