% Create training data for ANNs in BWE System 1.

clear all
clc


% -- Set up paths to extra tools.
path(path, [pwd '/voicebox']);
%path(path, [pwd '/neuralnet']);
path(path, [pwd '/misc']);


% -- Set up location, names and number of soundfiles.
snd_dir = '/svog/NST_data/trainset/tale/';
% snd_subdirs = {'ae4' 'aa3' 'ag6' 'a5i' 'adi' 'a8p' 'am5' 'afo' 'a5c' 'am1' 'aoj' 'a3x' 'aat' 'ace' 'abb' 'a1w' 'aah' 'akc' 'a84' 'abr'}; % 'aay' 'aou' 'ahq' 'a6t' 'ao7' 'a7b' 'ahg' 'ak9' 'ajp' 'aiv' 'ami' 'a2i' 'a7s' 'ahh' 'a6l' 'abx' 'ak3' 'a1x' 'a9x' 'ak0' 'alj' 'ahk' 'a00' 'a79' 'an7' 'a9u' 'abu' 'a76' 'akm' 'ao0' 'alh' 'a4a' 'acv' 'ahf' 'ajo' 'a7a' 'ac8' 'ac7' 'anp' 'agh' 'ald' 'a8s' 'acp' 'akv' 'a9j' 'amh' 'ao9' 'a7z' 'a73' 'ac0' 'ahw' 'amd' 'a6u' 'a9v' 'afs' 'aba' 'a25' 'a95' 'ahb' 'aev' 'a6w' 'abv' 'aip' 'a53' 'aj9' 'a5g' 'ani' 'abg' 'a4q' 'aif' 'ahm' 'a6i' 'aff' 'a0o' 'a5t' 'a6q' 'aj5' 'ah6' 'ago' 'a34'}; 100 speakers.
snd_subdirs = {'ae4', 'aa3', 'ag6', 'a5i', 'adi', 'a8p', 'am5', 'afo', 'a5c', 'am1', 'aoj', 'a3x', 'aat', 'ace', 'abb', 'a1w', 'aah', 'akc', 'a84', 'abr', 'aay', 'aou', 'ahq', 'a6t', 'ao7', 'a7b', 'ahg', 'ak9', 'ajp', 'aiv', 'ami', 'a2i', 'a7s', 'ahh', 'a6l', 'abx', 'ak3', 'a1x', 'a9x', 'ak0', 'alj', 'ahk', 'a00', 'a79', 'an7', 'a9u', 'abu', 'a76', 'akm', 'ao0'}; % 50 speakers.
snd_nums = [(18:37) (43:57) (63:77) (83:102) (108:122) (128:142)]; % 100 of the longest sentences, all different between speakers.
%i_random = randperm(length(snd_nums));
%snd_nums = snd_nums(i_random(1:50));
snd_ext = '.bin';

num_subdirs = length(snd_subdirs);
num_files = length(snd_nums);


savedir = 'train_data/20ms/';


% -- Set up parameters for the analysis.
F_sample = 16e3;
t_frame = 20e-3;

f_liftering = true; % Perform liftering?
k_lifter = 2; % Multiplication factor for lifters.

f_highband = false; % If CCs for only the high band is wanted.. (Bad idea.)
f_cconly = false; % If only CCs are wanted. (No GI or NFE).

num_delta_offset = 0; % How many previous and "future" frames to use for deltas.
                      % WARNING: INCOMPLETE IMPLEMENTATION! See below..


% -- Settings for normed frame energy.
N_minsearch = 400; % How many frames back in time to search for minimum energy.
alpha = 0.96; % Forgetting factor.

% Number of coefficients to use for analysis. Should have at least as many 
% cepstral coefficients as linear predictor coefficients! (C0 is not
% included.)
num_lpc_NB = 12; %
num_cc_NB = num_lpc_NB;
num_dcc_NB = num_lpc_NB; % How many CCs to use for deltas.


% For highband-only estimation, fewer LPCs are needed.
if f_highband
    num_lpc_WB = 8;
else
    num_lpc_WB = 18;
end

num_cc_WB = num_lpc_WB;



% -- Compute number of samples per frame, frame skip and overlap.
% The seemingly pointless division and multiplication is to ensure 50 % overlap, and
% that the skip length is divisible by two (due to interpolation in GetExcitation).
len_frame_NB = 4*round(F_sample/2 *t_frame/4);
len_skip_NB = len_frame_NB/2;

len_frame_WB = 2*len_frame_NB;
len_skip_WB = 2*len_skip_NB;



% -- Make windows for use in analysis.
win_NB = hamming(len_frame_NB)';
win_WB = hamming(len_frame_WB)';


% -- Make lifters.
L1 = k_lifter*(num_cc_NB+1);
L2 = k_lifter*(num_cc_WB+1);


if f_liftering
    lifter_NB = 1 + L1/2*sin((1:num_cc_NB)*pi/L1);
    lifter_WB = 1 + L2/2*sin((1:num_cc_WB)*pi/L2);
else
    lifter_NB = ones(1, num_cc_NB);
    lifter_WB = ones(1, num_cc_WB);
end



% -- Make filters. (Should be the same as in the BWE system!)
[b_pots, a_pots, pots_delay] = PotsFilter();
[b_hp, a_hp, hp_delay] = HighPassFilter();


% -- Initialize output buffer cells. (one cell per file)
cc_NB = cell(num_subdirs*num_files, 1);
c0_NB = cc_NB;
cc_WB = cc_NB;
gi_NB = cc_NB;
nfe_NB = cc_NB;
dc_NB = cc_NB;


% Allocate space for frame-counting.
num_frames = zeros(num_subdirs*num_files, 1);


% -- Get the cepstral coefficients for both narrow- and wideband signals
% from all files.
disp('Processing files...');
for i_subdir = 1:num_subdirs
%    filedir = [snd_dir snd_subdirs{i_subdir} '/'];
    filedir = [snd_dir '/'];
    
    % Load first file in subdirectory to estimate the noise level.
    filename = [snd_subdirs{i_subdir} '_' num2str(1, '%04d') snd_ext];
    snd_WB = binread([filedir filename]);

    snd_frames_WB = enframe(snd_WB, win_WB, len_skip_WB);
    frame_powers = mean(snd_frames_WB.^2, 2);
    
    % Define a power limit to act as speech detection.
    power_lim = mean(frame_powers)*10^(6/10);

    % -- Start on the files to make training data from.
    for i_file = 1:num_files
        i_cell = i_file + (i_subdir - 1)*num_files;

        disp(['File ' num2str(i_cell) ' of ' num2str(num_subdirs*num_files)]);
        
        % Construct filename and load it.
        filename = [snd_subdirs{i_subdir} '_' num2str(snd_nums(i_file), '%04d') snd_ext];
        snd_WB = binread([filedir filename]);

        len_snd_WB = length(snd_WB);
        
        if (len_snd_WB >= len_frame_WB)
            snd_frames_WB = enframe(snd_WB, win_WB, len_skip_WB);
            frame_powers = mean(snd_frames_WB.^2, 2); 

            % Find index of first and last frame identified as containing
            % speech.
            i_start = find(frame_powers >= power_lim, 1, 'first');
            i_end = find(frame_powers >= power_lim, 1, 'last');

            % Need some search space for normed frame energy.
            i_start = max(i_start-10, 1);
            i_end = min(length(frame_powers), i_end+10);

            % Discard samples before and after these frames.
            i_speech = (1 + (i_start - 1)*len_skip_WB):(len_frame_WB + (i_end - 1)*len_skip_WB);
            snd_WB = snd_WB(i_speech);
            disp(['    removed ' num2str(100*(1 - length(i_speech)/len_snd_WB)) ' % (leading/trailing silence)']);

            
            % Detect speech, perform sifting of analysis data afterwards.
            snd_frames_WB = enframe(snd_WB, win_WB, len_skip_WB);
            frame_powers = mean(snd_frames_WB.^2, 2); 
            i_speech = frame_powers >= power_lim;%*10^(6/10);


            % Create POTS and (possible) high band versions of the file.
            snd_NB = filter(b_pots, a_pots, snd_WB);
            snd_NB(1:pots_delay) = []; % Remove due to filter delay..
            snd_NB = downsample(snd_NB, 2);

            % If only the high band is wanted, perform highpass filtering here.
            if f_highband
                hisnd_WB = filter(b_hp, a_hp, snd_WB);
%                hisnd_WB(1:hp_delay) = []; % only 2 sample delay in 4-8 kHz..
            else
                hisnd_WB = snd_WB;
            end

            % Get the frames of both signals.
            snd_frames_NB = enframe(snd_NB, win_NB, len_skip_NB);
            hisnd_frames_WB = enframe(hisnd_WB, win_WB, len_skip_WB);

            % Calculate the number of frames. (One might have fewer than the
            % other!)
            num_frames(i_cell) = min([size(snd_frames_NB, 1) size(hisnd_frames_WB, 1)]);

            % Remove any excessive frames.
            snd_frames_NB = snd_frames_NB(1:num_frames(i_cell), :);
            hisnd_frames_WB = hisnd_frames_WB(1:num_frames(i_cell), :);
            i_speech(num_frames(i_cell)+1:end) = [];            
            
            % Start analyzing.
            disp('  Making analysis data');

            % Create LP and liftered LPCC for POTS and high band signal.
            lpc_NB = lpc(snd_frames_NB', num_lpc_NB);
            lpc_WB = lpc(hisnd_frames_WB', num_lpc_WB);
            
            temp_cc_NB = lpcar2cc(lpc_NB, num_cc_NB) .* repmat(lifter_NB, num_frames(i_cell), 1);
            temp_cc_WB = lpcar2cc(lpc_WB, num_cc_WB) .* repmat(lifter_WB, num_frames(i_cell), 1);
            
            % Discard silent frames.
            cc_NB{i_cell} = temp_cc_NB(i_speech, :);
            cc_WB{i_cell} = temp_cc_WB(i_speech, :);

            if ~f_cconly
                % Get gradient indexes and normed frame energy for
                % narrowband signal.
                temp_gi_NB = GradientIndex(snd_frames_NB);
                temp_nfe_NB = NormedFrameEnergy(snd_frames_NB, alpha, N_minsearch, 0);
                
                % Discard silent frames..?
                gi_NB{i_cell} = temp_gi_NB(i_speech, :);
                nfe_NB{i_cell} = temp_nfe_NB(i_speech, :);
            end

            num_frames(i_cell) = sum(i_speech);
            
            disp(['  Using ' num2str(100*sum(i_speech)/length(i_speech)) ' % of frames.']);

%             % If deltas are wanted.
%             if n_delta_offset > 0
%                 % Only do this if we have enough frames..
%                 if n_frames(i_cell) >= (1+2*n_delta_offset)
%                     disp('    Making delta coefficients.');
%                     
%                     % Get the excitation and compute c0.
%                     exc_NB = GetExcitation2(snd_NB, lpc_NB, n_skip_NB);
%                     exc_frames_NB = enframe(exc_NB, win_NB, n_skip_NB);
% 
%                     c0_NB{i_cell} = log(mean(exc_frames_NB.^2, 2))/2;
%                     
%                     % Compute deltas for some feature vector..
%                     if f_cconly
%                         temp_dc_NB = ComputeDeltas([c0_NB{i_cell} temp_cc_NB(:, 1:n_dcc_NB)], n_delta_offset);
%                         dc_NB{i_cell} = temp_dc_NB();
%                     else
%                         temp_dc_NB = ComputeDeltas([temp_gi_NB temp_nfe_NB c0_NB{i_cell} temp_cc_NB(:, 1:n_dcc_NB)], n_delta_offset);
%                         dc_NB{i_cell} = temp_dc_NB();
%                     end
%                     n_frames(i_cell) = n_frames(i_cell) - 2*n_delta_offset;
%                 else
%                     disp('    Not enough frames for delta coefficients, ignoring file.');
%                     n_frames(i_cell) = 0;
%                 end
%            end
            disp(' ');
        end
    end
end


% Preallocate input and output arrays.
num_frames_total = sum(num_frames);

if num_delta_offset > 0
    if f_cconly
        data_in = zeros(num_cc_NB + num_dcc_NB + 1, num_frames_total); % cc + dc0 + dcc
    else
        data_in = zeros(5 + num_cc_NB + num_dcc_NB, num_frames_total); % (gi + nfe) + cc + (dgi + dnfe + dc0) + dcc
    end
else
    if f_cconly
        data_in = zeros(num_cc_NB, num_frames_total);
    else
        data_in = zeros(2 + num_cc_NB, num_frames_total);
    end
end

data_out = zeros(num_cc_WB, num_frames_total);

num_cells = num_subdirs*num_files;


% Initialize frame index.
i_frames = 0;

% Concatenate data from the files.
disp('Concatenating data..');
for i_cell = 1:num_cells
    disp(['    Cell #' num2str(i_cell) ' of ' num2str(num_cells)]);
    % File must have more than 0 frames to be used. Otherwise it was missing, or had too few frames for computation of delta coefficients.
    if num_frames(i_cell) > 0
        i_frames = i_frames(end) + (1:num_frames(i_cell));
        
        if num_delta_offset > 0
%                 data_in(:, i_frames) = [gi_NB{i_cell}(1+num_delta_offset:end-num_delta_offset) ...
%                                         nfe_NB{i_cell}(1+num_delta_offset:end-num_delta_offset) ...
%                                         cc_NB{i_cell}(1+num_delta_offset:end-num_delta_offset, :) ...
%                                         dc_NB{i_cell}]';
        else 
                data_in(:, i_frames) = [gi_NB{i_cell} ...
                                        nfe_NB{i_cell} ...
                                        cc_NB{i_cell}]';
        end
    
        data_out(:, i_frames) = cc_WB{i_cell}(1+num_delta_offset:end-num_delta_offset, :)';
    end
end

% Split the dataset into training and verification parts. (Make array of indices)
num_dataset = size(data_in, 2);
num_trainset = round(2/3*num_dataset);

i_random = randperm(num_dataset);
i_train = i_random(1:num_trainset);
i_verif = i_random((num_trainset+1):end);


% Make filename for savefiles.
filename = [num2str(num_subdirs) 'sp_' ];

if ~f_cconly
    filename = [filename 'ginfe_'];
end

if f_highband
    filename = [filename 'hi_'];
end

% Add descriptors for options to filename.
if num_delta_offset > 0
    filename = [filename num2str(num_delta_offset) 'D_'];
else
    filename = [filename 'noD_'];
end

if ~f_liftering
    filename = [filename 'noL_'];
else
    filename = [filename num2str(k_lifter) 'L_'];
end

filename = [filename num2str(num_lpc_NB) 'LP_to_' num2str(num_cc_NB) 'CC_'];
filename = [filename num2str(num_lpc_WB) 'LP_to_' num2str(num_cc_WB) 'CC'];

% Save directly to binary to conserve a bunch of space.
disp(['Saving data to: ' savedir filename '*']);
SaveBinaryData([savedir filename], data_in, data_out, i_train, i_verif);
