function [lpc_out] = CcToLpc(cc, len_frame, num_lpc)
% Compute stable LPCs from a given set of cepstral coefficients.
% This method relies on the DFT and Levinson-Durbin algorithm.
%
% Input arguments:
% cc : cepstral coefficients, as row vectors
% len_frame : number of samples in frame from which cc stems
% num_lpc : number of linear prediction coefficients to compute
%
% Output:
% lpc_out : linear prediction coefficients, as row vectors

num_rows = size(cc, 1);

% Use a DFT of length equal to the autocorrelation for frames 
% of length len_frame amples.
len_fft = 2*len_frame-1;

CC = fft([zeros(num_rows,1) cc]', len_fft);
H = exp(CC);
r = ifft(abs(H).^2, len_fft);

lpc_out = levinson(r, num_lpc);
