function [mu, sigma, num_samples] = CellStats(cells)
% Compute mean and standard deviation of a cell array, where each cell
% contains a one dimensional vector.
% Also returns the total number of samples.
%
% Input arguments
% cells : array of cells, each containing a one-dimensional vector of
%         any length.
%
% Output
% mu : sample mean of all the vector elements in the cells
% sigma : sample standard deviation of the same
% num_samples : total number of vector elements


num_cells = length(cells);
num_samples = 0;
mu = 0;

for i_cell = 1:num_cells
    num_samples = num_samples + length(cells{i_cell});
    mu = mu + sum(cells{i_cell});
end

mu = mu/num_samples;

sigma = 0;

for i_cell = 1:num_cells
    sigma = sigma + sum((cells{i_cell} - mu).^2);
end

sigma = sqrt(sigma/(num_samples-1));
