function [y] = ComputeDeltas(x, num_offset)
% Computes delta coefficients for a set of row vectors, using
% linear regression.
% Note that the output will contain less rows than the input.
%
% Input arguments
% x : collection of row vectors
% num_offset : span of memory (vectors into the past and future)
%
% Output
% y : row vectors containing deltas for x

[num_rows, num_cols] = size(x);
y = zeros(num_rows - 2*num_offset, num_cols);
weights = repmat((1:num_offset)', 1, num_cols);

for i_row = (1+num_offset):(num_rows-num_offset)
	y(i_row-num_offset, :) = sum(weights.* ...
        (x(i_row+(1:num_offset), :) - x(i_row-(1:num_offset), :)), 1)./ ...
        sum(weights.^2, 1)/2;
end
