function [e_WB] = ExtendExcitationHigh(e_NB)
% Extend POTS excitation into the high band by use of modulation.
%
% Input arguments
% e_NB : excitation with 8 kHz sample rate
%
% Output
% e_WB : high band excitation with 16 kHz sample rate

% Create needed filters.
[b_aa, a_aa, aa_delay] = AntiAliasingFilter();
[b_hp, a_hp, hp_delay] = HighPassFilter();

% Upsample and filter away aliased components.
temp = filter(b_aa, a_aa, 2*upsample(e_NB, 2));

% Modulate with 3.4 kHz.
temp = 2*temp.*sin(2*pi*3.4/16*(0:length(temp)-1));

% High pass filter.
e_WB = filter(b_hp, a_hp, temp);
