function [gains] = GainCorrection(lpc_orig, lpc_est, a_est)
% Compute gain correction to get similar magnitude response in the 
% POTS band for two filters.
%
% Input arguments
% lpc_orig : reference LP-filters
% lpc_est : LP-filters to perform gain correction of
% a_est : in case an all-zero filter is needed to correct response of lpc_est
%         (such as de-emphasis)
%
% Output
% gains : correction gains for each LP-filter in lpc_est

% If a_est is not specified, default to 1.
if ~exist('a_est', 'var')
    a_est = 1;
end

% Count number of frames and allocate space for gain.
num_frames = size(lpc_est, 1);
gains = zeros(num_frames, 1);

% Frequency band to compare gain in. (500 - 3.2k Hz, with 50 Hz resolution)
F_gain = 5e2:50:3.2e3;

% Compute correction gain per frame.
for i_frame = 1:num_frames
        gains(i_frame) = ...
            mean(abs(freqz(1, lpc_orig(i_frame, :), F_gain, 16e3))) / ...
            mean(abs(freqz(a_est, lpc_est(i_frame, :), F_gain, 16e3)));
end
