function [e] = GetExcitation2(x, x_lpc, len_skip)
% Extract the excitation from a signal, using 2x interpolation of
% supplied LP-filters.
%
% Input arguments
% x : signal to extract excitation from
% x_lpc : LP-filters to use
% len_skip : number of samples between analysis windows (should be divisible by 4!), assume frame length is twice the skip length
%
% Output
% e: excitation signal

% Allocate space for excitation, count number of frames.
e = zeros(size(x));
num_frames = size(x_lpc, 1);

% No interpolation for the first 2.5 subframes in first frame.
i_samples = 1:(len_skip*5/4);
[e(i_samples), z_line] = filter(x_lpc(1, :), 1, x(i_samples));

% Perform interpolated filtering on the rest of the frames
for i_frame = 2:num_frames-1
    % Interpolate LP coefficients for first subframe
    lpc_curr = 0.5*(x_lpc(i_frame, :) + x_lpc(i_frame - 1, :));
    
    % Filter each subframe.
    for i_subframe = 1:2
        i_samples = (1:len_skip/2) + len_skip*(i_frame - 1) + len_skip/2*(i_subframe - 1/2);
        [e(i_samples), z_line] = filter(lpc_curr, 1, x(i_samples), z_line); 

        % No interpolation for second subframe.
        lpc_curr = x_lpc(i_frame, :);
    end
end


% The last frame also requires some special handling.
% Interpolation for first subframe only.
lpc_curr = 0.5*(x_lpc(num_frames, :) + x_lpc(num_frames - 1, :));

i_samples = (1:len_skip/2) + len_skip*(num_frames - 1) + len_skip/4;
[e(i_samples), z_line] = filter(lpc_curr, 1, x(i_samples), z_line);

% No interpolation for the last 2.5 subframes.
i_samples = (1:(len_skip*5/4)) + len_skip*(num_frames - 1) + 3/4*len_skip;
e(i_samples) = filter(x_lpc(num_frames, :), 1, x(i_samples), z_line);
