function [y] = GmmRegress(GMM, x)
% Estimate y from x, using GMM-based regression.
%
% Input arguments
% GMM : Gaussian mixture model to use for regression
% x : collection of row vectors
%
% Output:
% y : collection of regressed row vectors


% Count number of rows, dimensions of x and y and number of mixes in GMM.
[num_rows, num_dimx] = size(x);
num_dimy = size(GMM.muy, 1);
num_mixes = length(GMM.apriori);

% Allocate space for output and temporary probability computations.
y = zeros(num_rows, num_dimy);
prob = zeros(num_mixes, 1);

% For each input vector:
for i_row = 1:num_rows
    % Compute the probability density of x for each mixture.
    for i_mix = 1:num_mixes
        temp = x(i_row,:)' - GMM.mux(:,i_mix);
        prob(i_mix) = GMM.apriori(i_mix)/GMM.constxx(i_mix)*exp(-0.5*temp'*GMM.invCxx(:,:,i_mix)*temp);
    end
    
    % Normalize probability densities.
    prob = prob/sum(prob);
    
    % Sum "regressed" y-values from the mixtures, weighted with their probability.
    for i_mix = 1:num_mixes
        y(i_row,:) = y(i_row,:) + ...
            prob(i_mix)*(GMM.muy(:,i_mix) + GMM.Cyx(:,:,i_mix)*GMM.invCxx(:,:,i_mix)*(x(i_row,:)' - GMM.mux(:,i_mix)))';
    end
end
