function [gradindex] = GradientIndex(input)
% Compute the gradient index for a set of analysis frames.
%
% Input arguments
% input : analysis frames as a collection of row vectors
%
% Output
% gradindex : vector with the gradient index for each frame

n_rows = size(input, 1);

gradindex = zeros(n_rows, 1);

for i_row = 1:n_rows
    signal = input(i_row, :);
    
    delta = diff(signal);
    psi = diff(sign(delta));

    gradindex(i_row) = sum(abs(delta(2:end)).*abs(psi/2))/sqrt(sum(signal.^2));
end
