function [x_in, x_out] = LoadBinaryData(filename)
% Load binary training/verification data.
% Works with data saved by SaveBinary(...)
%
% Input arguments
% filename : name of file to load binary data from
%
% Output
% x_in : input data as column vectors
% x_out : output data as column vectors

% Structure of datafiles
% uint32 - number of training vectors (input and output vector pairs)
% uint32 - number of input parameters
% uint32 - number of output parameters
% double - input vectors in sequence
% double - output vectors in sequence

% Read binary data.
fid = fopen(filename, 'r');

n_setsize = fread(fid, 1, 'uint32');
n_inputs = fread(fid, 1, 'uint32');
n_outputs = fread(fid, 1, 'uint32');

x_in = fread(fid, [n_inputs, n_setsize], 'double');
x_out = fread(fid, [n_outputs, n_setsize], 'double');

fclose(fid);
