function [DCT] = MakeDCT(dim_x, dim_y)
% Create a type 2 DCT with given dimensions.
% For perfect recreation to be possible, dim_y = dim_x.
%
% Input arguments
% dim_x : dimensions of input vector
% dim_y : either dimensions of output vector, or a vector containing warping of n in the DCT
%
% Output
% DCT : DCT matrix with specified dimensions

if length(dim_y) == 1
    DCT = zeros(dim_y, dim_x);

    k = 0:dim_x-1;

    for n = 0:dim_y-1
        DCT(n+1, :) = sqrt(2/dim_x) * cos( (2*k+1)*n*pi/2/dim_x );
    end
else
    DCT = zeros(length(dim_y), dim_x);
    
    k = 0:dim_x-1;
    
    for i_n = 1:length(dim_y)
        DCT(i_n, :) = sqrt(2/dim_x) * cos( (2*k+1)*dim_y(i_n)*pi/2/dim_x );
    end
end
