function [IDCT] = MakeIDCT(dim_x, dim_y)
% Create type 3 DCT (inverse to type 2).
% For perfect recreation, dim_y = dim_x
% 
% Input arguments
% dim_x : dimensions of input vector
% dim_y : either dimensions of output vector, or a vector containing warping of n in the IDCT
%
% Output
% IDCT : inverse DCT matrix with specified dimensions

if length(dim_y) == 1

    IDCT = zeros(dim_y, dim_x);

    n = 1:dim_x-1;

    for k = 0:dim_y-1
        IDCT(k+1, :) = sqrt(2/dim_x)*[1/2 cos( (2*k+1)*n*pi/2/dim_x )];
    end

else

    IDCT = zeros(length(dim_y), dim_x);
    
    n = 1:dim_x-1;
    
    for i_k = 1:length(dim_y)
        IDCT(i_k, :) = sqrt(2/dim_x)*[1/2 cos( (2*dim_y(i_k)+1)*n*pi/2/dim_x )];
    end
end
