function [NFE] = NormedFrameEnergy(frames, alpha, N_min, energy_mem)
% Compute normed frame energy for a set of analysis frames.
%
% Input arguments
% frames : analysis frames as a collection of row vectors
% alpha : forgetting factor for energy memory
% N_min : span of minimum energy search window
% energy_mem : initializing value for energy memory
%
% Output
% NFE : vector with the normed frame energy for each frame

% Count number of frames, preallocate output variable.
num_frames = size(frames, 1);
NFE = zeros(num_frames, 1);

% Compute frames energies, initialize energy_mem to mean if set to 0
energies = sum(frames.^2, 2);
if energy_mem == 0
    energy_mem = mean(energies);
end

% Compute normed frame energy for each frame.
for i_frame = 1:num_frames
    energy_min = min( energies( i_frame:-1:max([1 (i_frame - N_min)]) ) );
    energy_mem = alpha * energy_mem + (1 - alpha) * energies(i_frame);
    NFE(i_frame) = (log(energies(i_frame)) - log(energy_min))/(log(energy_mem) - log(energy_min));
end
