function [] = SaveBinary(filename, data_in, data_out, i_train, i_verif)
% Save training and verification data to a binary file for use with
% the neurotrain commandline tool.
% Two files are created, with the postfixes _train.dat and _verif.dat
% 
%
% Input arguments
% filename : base filename to save data to
% data_in : input vectors to the ANN, as column vectors
% data_out : output vectors for the ANN, as column vectors
% i_train : indexes of training vectors
% i_verif : indexes of verification vectors

n_inputs = size(data_in, 1);
n_outputs = size(data_out, 1);
n_trainset = length(i_train);
n_verifset = length(i_verif);


% -- Make training file.
disp('Saving training file.');

savefile = [filename '_train.dat'];
fileid = fopen(savefile, 'w');

% Write size of dataset.
fwrite(fileid, n_trainset, 'uint32');
fwrite(fileid, n_inputs, 'uint32');
fwrite(fileid, n_outputs, 'uint32');

% These are stored column wise, i.e. one vector after another.
fwrite(fileid, data_in(:, i_train), 'double');
fwrite(fileid, data_out(:, i_train), 'double');

fclose(fileid);


% -- Make verification file.
disp('Saving verification file.');

savefile = [filename '_verif.dat'];
fileid = fopen(savefile, 'w');

fwrite(fileid, n_verifset, 'uint32');
fwrite(fileid, n_inputs, 'uint32');
fwrite(fileid, n_outputs, 'uint32');

fwrite(fileid, data_in(:, i_verif), 'double');
fwrite(fileid, data_out(:, i_verif), 'double');

fclose(fileid);
