function [] = SaveBinaryData_GMM(filename, data_in, data_out)
% Save training data to a binary file for use in GMM training.
% Must be converted using the savetrain commandline tool for
% HTK to be able to read it.
%
%
% Input arguments:
% filename : name of file to save data to
% data_in : input (in regression) data as column vectors
% data_out : output (in regression) data as column vectors

[num_inputs, num_vectors] = size(data_in);
num_outputs = size(data_out, 1);



% -- Make training file.
disp('Saving training file.');

savefile = filename;
fileid = fopen(savefile, 'w');

% Output size of dataset.
fwrite(fileid, num_vectors, 'uint32');
fwrite(fileid, num_inputs + num_outputs, 'uint32');

% These are stored column wise, i.e. one vector after another.
% The order is [input; output]
for idx_vector = 1:num_vectors
    fwrite(fileid, data_in(:, idx_vector), 'double');
    fwrite(fileid, data_out(:, idx_vector), 'double');
end

fclose(fileid);
