function [] = ShowSpectro(sig, win, len_olap, len_fft, F_sample, loglim, cmap, plotlim)
% This function simply shows a spectrogram of an input signal.
%
% Input arguments
% sig : signal to analyze
% win : vector containing window
% len_olap : number of overlapping samples of analysis windows
% len_fft : how large DFT to use
% F_sample : sample rate
% loglim : how many dB below maximum to use as floor
% cmap : colormap
% plotlim : axis limits for time and frequency

[S, F, T, P] = spectrogram(sig, win, len_skip, len_fft, F_sample);

P_log = 20*log10(abs(S));
maxP_log = max(max(P_log));

P_log = P_log - maxP_log;

P_log = max(P_log, -loglim(1));

i_T = (T >= plotlim(1)) & (T <= plotlim(2));
i_F = (F >= plotlim(3)) & (F <= plotlim(4));

%surf(T(i_T), F(i_F), P_log(i_F,i_T), 'EdgeColor', 'None');
%axis(plotlim);

% Have to use imagesc for later output because Matlab is bugridden.
imagesc(T(i_T), F(i_F), P_log(i_F, i_T));
set(gca,'YDir','Normal');

%view([0 0 1]);
%shading('interp');

colormap(cmap);
set(gca,'FontName','Times');
xlabel('Time [s]');
ylabel('Frequency [Hz]');
