clear

path(path, [pwd '/fann_bind']);
path(path, [pwd '/voicebox']);
path(path, [pwd '/misc']);


% -- Define location of input and output sound files.
snd_dir = '/svog/NST_data/testset/tale/';
%snd_savedir = 'outwavs/';


% -- Configure GMMs for MFCC and gain estimation.
gmmfile2 = 'gmms/sep_mfcc_5d_mix_16'; num_delta_offset = 5; 
gmmfile3 = 'gmms/sep_gain_5d_mix_16';

num_mfcc = [10 6 6 4]; % Define GMM dimensionality: <static lowband MFCC> <dynamic lowband MFCC> <static highband MFCC> <dynamic highband MFCC>

% -- Define which system to analyze.
f_sys2mem = true;
f_savefiles = false; % Save output files?

% -- Setup of analysis.
F_sample = 16e3;
num_lpc = 18; % Should be the same as for the systems!


% 50 random speakers.
speakers = {'st17_se12', 'st24_se14', 'st23_se14', 'st25_se17', 'st16_se12', 'st26_se04', 'st24_se06', 'st25_se04', 'st24_se03', 'st25_se09', 'st24_se19', 'st26_se06', 'st23_se31', 'st25_se10', 'st17_se04', 'st25_se08', 'st24_se11', 'st24_se16', 'st25_se12', 'st16_se10', 'st24_se10', 'st25_se18', 'st25_se07', 'st17_se08', 'st24_se08', 'st16_se04', 'st23_se12', 'st17_se02', 'st25_se14', 'st25_se22', 'st17_se09', 'st24_se12', 'st23_se17', 'st23_se28', 'st26_se05', 'st17_se06', 'st16_se08', 'st23_se10', 'st26_se08', 'st24_se18', 'st23_se07', 'st26_se03', 'st23_se15', 'st16_se06', 'st17_se07', 'st24_se07', 'st24_se04', 'st25_se11', 'st24_se02', 'st17_se11'}; % 50 speakers
%speakers = {'st17_se12'};
% 10 files, differing across speakers.

files = [619:626 638:644]; % 15 files
%files = 619;

num_files = length(speakers)*length(files);


% -- Load GMMs and linear map
if f_sys2mem
    GMMmem = LoadHtkGmm(gmmfile2, sum(num_mfcc(1:2)), sum(num_mfcc(3:4)));
    GMMmem_gain = LoadHtkGmm(gmmfile3, sum(num_mfcc(1:2)), 1); 
end

% Allocate space for log spectral deviation measures.
lsd_sys2mem = cell(num_files, 1);

rmslsd_sys2mem = lsd_sys2mem;


% -- Create analysis window and filters.
len_win = 2*round(F_sample/2*20e-3);
win = hamming(len_win);

[b_pots, a_pots, pots_delay] = PotsFilter();
a_preemph = [1 0.97];


% -- Select some files to save for listening tests.
num_savefiles = 10;

%temp = randperm(num_files);
%i_savefiles = temp(1:num_savefiles);
i_savefiles = (1:round(400/num_savefiles):400);


% Initialize pointer for storing LSD measures.
i_row = 1;

for s_speaker = speakers
    % Load first file to estimate noise level.
    snd_subdir = [s_speaker{1} '/'];

    snd_file = [s_speaker{1} '_' num2str(1, '%03d') '.raw'];
    snd = rawread([snd_dir snd_subdir snd_file]);
    
    snd_frames = enframe(snd, win, len_win/2);
    snd_power = mean(snd_frames.^2, 2);

    % Define a power limit to define frames as speech.
    % Only speech frames are used in LSD computations!
    power_lim = mean(snd_power)*16; % 16 = ~12 dB


    % Start on the files..
    for i_file = files
        disp(['File #' num2str(i_row) ' of ' num2str(num_files)]);
    
        % Construct filename and load.
        snd_file = [snd_subdir(1:end-1) '_' num2str(i_file, '%03d') '.raw'];
        snd = rawread([snd_dir snd_subdir snd_file])';
   
        % Compute correct LPC-filters and frame powers.
        snd_frames = enframe(snd, win, len_win/2);
        lpc_orig = lpc(snd_frames', num_lpc);
        snd_power = mean(snd_frames.^2, 2);
        
        % Find the frames which contain speech.
        i_speech = snd_power >= power_lim;
        
        % Create POTS version, and downsampled POTS version.
        snd_pots = filter(b_pots, a_pots, snd);
        snd_pots(1:pots_delay) = [];
        snd_pots2 = downsample(snd_pots, 2);


        % -- Analysis for the flagged systems starts here.
        % Note that all analysis is done on spectral envelopes
        % related to frames detected as speech only!

        % If analysis of system 2 with memory (GMM) is flagged.. 
if f_sys2mem
        [y_sys2mem, lpc_sys2mem] = sep_System2Synth(snd_pots, GMMmem, GMMmem_gain, num_delta_offset, num_mfcc);
    
        % Must compensate for delta offset!!
        num_frames = size(lpc_sys2mem, 1);
        i_temp = i_speech(1+num_delta_offset:num_frames+num_delta_offset);

        i_temp2 = i_speech;
        i_temp2(1:num_delta_offset) = false;
        i_temp2(num_frames+num_delta_offset+1:end) = false;
        
        gain_sys2mem = GainCorrection(lpc_orig(i_temp2, :), lpc_sys2mem(i_temp, :), a_preemph);

        lsd_sys2mem{i_row} = LogSpectralDeviation(lpc_orig(i_temp2, :), lpc_sys2mem(i_temp, :), gain_sys2mem, a_preemph);
        
        rmslsd_sys2mem{i_row} = RMSLogSpectralDeviation(lpc_orig(i_temp2, :), lpc_sys2mem(i_temp, :), gain_sys2mem, a_preemph);
end

        % Save listening examples if this is one of the random save files.
        if (f_savefiles && sum(i_row == i_savefiles))
            % Save the original and POTS version.
            wavwrite(snd, F_sample, 16, [snd_savedir num2str(i_row) '_orig.wav']);
            wavwrite(snd_pots, F_sample, 16, [snd_savedir num2str(i_row) '_pots.wav']);
            
            if f_sys2mem
                wavwrite(y_sys2mem, F_sample, 16, [snd_savedir num2str(i_row) '_sys2mem_sep.wav']);
            end
        end

        % Update row pointer.
        i_row = i_row + 1;
    end
end

% Output information about which systems were analyzed.
if f_sys2mem
	disp(['GMM 2: ' gmmfile2]);
end
disp(' ');


% Output LSD measurement results.
disp('LSD:');
if f_sys2mem
    [mu, sigma, samples] = CellStats(lsd_sys2mem);
	disp(['sys2mem: ' num2str(mu) ' dB, with std.dev: ' num2str(sigma) ' from ' num2str(samples) ' frames']);
end
disp(' ');

disp('RMS-LSD:');
if f_sys2mem
    [mu, sigma] = CellStats(rmslsd_sys2mem);
	disp(['sys2mem: ' num2str(mu) ' dB, with std.dev: ' num2str(sigma)]);
end
