#!/usr/bin/env python
#
# Train a GMM using HTK.
#
#

import sys
import os

def writehedf(fn, incr):
    hedf = open(fn, "w")
    hedf.write("MU +" + str(incr) + " {" + outfn + ".state[2].mix}\n")
    hedf.close()
    
def copyHMMdef(oldfn, newfn):
    inf = open(oldfn, "r")
    ouf = open(newfn, "w")

    lines = inf.readlines()
    inf.close()
    for l in lines:
        if l[0:2] == "~h":
            ouf.write("~h \"" + newfn + "\"\n")
        else:
            ouf.write(l)

    ouf.close()


if len(sys.argv) < 6 or len(sys.argv) > 8:
    print 'usage: ' + sys.argv[0] + ' [-a] [-d] [-2] vecsize feattype nummix list_of_train_files outfile(no_path)'
    print '           -a   store all trained GMM (with all number of mixtures)'
    print '           -d   use diagonal covariance matrix'
    print '           -2   double the number of mixtures per iteration'
    sys.exit(-1)

storeall = False
diagcov = False

pcount = 1
if (sys.argv[pcount] == "-a"):
    storeall = True
    pcount = pcount + 1

if (sys.argv[pcount] == "-d"):
    diagcov = True
    pcount = pcount + 1

if (sys.argv[pcount] == "-2"):
    doublemix = True
    pcount = pcount + 1
else:
    doublemix = False

vecsize = int(sys.argv[pcount])
pcount = pcount + 1
feattype = sys.argv[pcount]
pcount = pcount + 1
nummix = int(sys.argv[pcount])
pcount = pcount + 1
filelist = sys.argv[pcount]
pcount = pcount + 1
outfn = sys.argv[pcount]

tmpdir  = '_tmp' + str(os.getpid())
protfn  = tmpdir + '/' + outfn
hedfn   = tmpdir + '/hed'
hedlist = tmpdir + '/hedlist'

# create temporary dir
os.mkdir(tmpdir)

# write prototype file
prf = open(protfn, "w")
if diagcov:
    prf.write('~o <VECSIZE> ' + str(vecsize) +  '<NULLD><' + feattype + '><DIAGC>\n')
else:
    prf.write('~o <VECSIZE> ' + str(vecsize) +  '<NULLD><' + feattype + '><FULLC>\n')
prf.write('~h \"' + outfn + '\"\n')
prf.write('<BEGINHMM>\n')
prf.write(' <NUMSTATES> 3\n')
prf.write(' <STATE> 2\n')
prf.write('  <MEAN> ' + str(vecsize) + '\n')
prf.write('   ')
for i in range(vecsize):
    prf.write(' 0.0')
prf.write('\n')
if diagcov:
    prf.write('  <VARIANCE> ' + str(vecsize) + '\n')
    prf.write('   ')
    for i in range(vecsize):
        prf.write(' 1.0')
    prf.write('\n')
else:
    prf.write('  <INVCOVAR> ' + str(vecsize) + '\n')
    prf.write('   ')
    for i in range(vecsize):
        for j in range(vecsize-i):
            if j == 0:
                prf.write(' 1.0')
            else:
                prf.write(' 0.0')
        if i < vecsize-1:    
            prf.write('\n')   
    prf.write('\n')
prf.write(' <TRANSP> 3\n')
prf.write('  0.0 1.0 0.0\n')
prf.write('  0.0 0.5 0.5\n')
prf.write('  0.0 0.0 0.0\n')
prf.write('<ENDHMM>\n')
prf.close()

# Initialize using HInit
cmdline = 'HInit -v 0.01 -m 1 -S ' + filelist + ' ' + protfn
print 'Initializing single mixture'
print cmdline
os.system(cmdline)

# if storeall, then copy HMM file to fn_mix_1
# and change ~h entry of HMM def file
if storeall:
    curfn = outfn + '_mix_1'
    copyHMMdef(outfn, curfn)

# Add one mixture and retrain until enough mixtures

# write cmd file and list file for HHed
writehedf(hedfn, 1)
hlfn = open(hedlist, "w")
hlfn.write(outfn + "\n")
hlfn.close()
curnum  = 1
curincr = 1
count   = 1
while curnum < nummix:
    # if doubling per training is wanted.
    if doublemix:
        curincr = curnum

    # check if curincr will make mix greater than target mix.
    if (curnum + curincr) > nummix:
        curincr = nummix - curnum

    newnum = curnum + curincr

    # write new HHEd file.
    os.unlink(hedfn)
    writehedf(hedfn, curincr)

    #print 'curnum = ' + str(curnum) + ', newnum = ' + str(newnum) + ', curincr = ' + str(curincr) + ', count = ' + str(count)
    
    print 'Training GMM with ' + str(newnum) + ' mixtures'
    cmdline = 'HHEd ' + hedfn + ' ' + hedlist
    print cmdline
    os.system(cmdline)
    cmdline = 'HRest -v 0.01 -m 1 -S ' + filelist + ' ' + outfn
    print cmdline
    status = os.system(cmdline)
    if status != 0:
        break

    count = count + 1

    curnum = newnum

    # if storeall, then copy HMM file to fn_mix_NNN
    # and change ~h entry of HMM def file
    if storeall:
        curfn = outfn + '_mix_' + str(curnum)
        copyHMMdef(outfn, curfn)

# if storeall, delete output file, since this has already been stored under
# a different name
if storeall:
    os.unlink(outfn)

# delete temporary files
os.unlink(hedlist)
os.unlink(hedfn)
os.unlink(protfn)

# remove temporary dir
os.rmdir(tmpdir)
